#----------------------------------------------------------------------------
#                     BLiP function
#                     version 1.0
# updated 9/27/94: add freqtext so that when freq=0, it won't write "0"
# updated 1/2/97: add text.cex
# updated 4/5/99: comment out ylab!=blip2
# updated 4/27/01: replace return(x=x, y=y) with return(x, y)
# updated 8/20/09: converted to R
# updated 10/20/09: add the orientation selection
# updated 4/15/10: add the line.percentile and line.percentile.length options
# updated 6/14/18: add different colors across groups
#updated 6/14/18: removed ylab requirement
#---------------------------------------------------------------------------

#' Draws one-dimensional distribution plots.
#' Blip can be used to construct standard or customized distribution plots.
#'
#' @export
#' @param ... Vectors or lists containing numeric components(e.g. the output of split) to be plotted.
#' Missing values(NAs) are allowed.
#' @param std.plot This option overrides all other related graphical parameters.
#' std.plot="boxplot" draws conventional boxplots;
#' std.plot="histogram" produces histogram;
#' std.plot="polygon" plots frequency polygon.
#' @param graph.region A numerical vector with 2 elements which specifies the lower and upper graphical region between 0 and 1.
#' @param graph.type A character value indicating the reference line.
#' "c" for centered and "b" for baseline.
#' @param graph.width A character value indicating the type of width of the box.
#' "f" for fixed and "v" for variable.
#' @param graph.uniform A logical value(T/F) to indicate the choice of uniform/non-uniform height on y-axis across groups.
#' @param text.cex A number to control the size of the text and symbols relative to the default.
#' @param nclass The smoothing parameter which corresponds to how many classes the total number of observations will be divided.
#' @param box.p A numeric vector of percentiles ranging from 0 to 1.
#' Missing values are not allowed in the vector except box.p=NA.
#' Numeric value 0 can be used as a separator for separatingboxes.
#' Each box must contain at least two percentiles.
#' @param box.p.bar A vector of logical value indicating whether to draw the vertical bars on percentiles.
#' @param box.label.on A Logical value indicating whether to label the percentiles on the boxes.
#' @param box.label.cex A numeric value to control the size of the labels.
#' @param box.label.above A numeric value indicating the distance of the labels above the box.
#' @param box.density A numeric value to indicate the density of the box shading lines.
#' @param box.angle A numeric value to indicate the angle of the box shading lines.
#' @param box.col A vector to indicate the color(s) of the box shading lines.
#' @param line.extend.p A numeric vector of percentiles ranging from 0 to 1, indicating segments of lines to be drawn outside the boxes.
#' @param line.percentile A numeric vector of percentiles ranging from 0 to 1, indicating segments of lines to be drawn outside the boxes.
#' @param line.percentile.length A numeric value indicating the length of the line.
#' @param line.percentile.col A numeric value indicating the color of the line.
#' @param line.percentile.width A numeric value indicating the width of the line.
#' @param line.extend.sd A numeric value indicating the length of lines to be drawn from the mean in standard deviation.
#' @param line.extend.se A numeric value indicating the length of lines to be drawn from the mean in standard error.
#' @param mean.pch A numeric value to specify the symbol for marking the location of the mean.
#' mean.pch=4 corresponds to "x".
#' @param mean.cex A numeric value to specify the size of mean symbol.
#' @param mean.col A numeric value to specify the color of mean symbol.
#' @param line.extend.col A numeric value to indicate the color of the line.
#' @param line.extend.width A numeric value to indicate the thickness of the line.
#' @param point.pattern When point.pattern="on-line", the points outside the boxes will be displayed on the reference line.
#' When point.pattern="stacking", the points outside the boxes will be separated by a fixed distance if there are more than one points with the same value.
#' When point.pattern="evenly-spaced", the points outside the boxes will be evenly spreaded out if there are more than one points with the same value.
#' When point.pattern="jittered", the points outside the boxes will be displayed randomly within the graph width.
#' When point.pattern="max-range", the points outside the boxes will be drawn at the maximum height of that point.
#' When point.pattern="vertical-bar", a vertical bar instead of a point will be drawn at the location of each observation.
#' @param point.type A character value to indicate whether a point("p"),a line("l"), both("b"), or none("n") will be drawn.
#' @param point.cex A numeric value to specify the size of points
#' @param point.pch A numeric or character value to specify the shape of points.
#' @param point.col A numeric value to specify the color of the points.
#' @param xlim A numeric vector consists of 2 elements which indicates the range of x-axis.
#' @param axes A logical value to indicate whether or not to plot axes.
#' @param xaxt "s"=standard x-axis plotting,"n"=null x-axis plotting.
#' @param xlab Labels on x-axis.
#' @param ylab Labels on y-axis.
#' @param breaks A numeric vector indicating break points for histogram and polygon.
#' @param seed An integer. This is a seed for the random number generator which is used in point.pattern="jittered".
#' @param orientation Orientation of the graph. Options are "horizontal" and "vertical".
#' @param vertical.xstart The side the x-values start from when the orientation of the graph is vertical.
#' "left" displays x-values in ascending order; "right" displays x-values in descending order.
#' @param violin.bp Indicates whether to draw a boxplot over the violin plot. Ignored if std.plot != "violin"
#' @param violin.sp Indicates whether to draw a scatterplot over the violin plot. Ignored if std.plot != "violin"
#' @param violin.bp.region Indicates upper and lower limits of a violin boxplot.
#'
#' @examples
#' x <- rnorm(100)
#' y <- c(rep(1, 50), rep(2, 50))
#' blip(x, std.plot = "boxplot")
#' blip(x, graph.type = "b", graph.width = "v", box.p = (0:10) / 10)
blip <- function(..., std.plot = "", graph.region = c(.25, .75), graph.type = "c",
               graph.width = "f", graph.uniform = T, text.cex = 1, nclass = 5,
               box.p = c(.25, .5, .75), box.p.bar = rep(T, length(box.p)),
               box.label.on = T, box.label.cex = par("cex"), box.label.above = .05,
               box.density = 0, box.angle = 45, box.col = c(par("col")),
               line.extend.p = NA, line.percentile = NA, line.percentile.length = 6,
               line.percentile.col = par("col"), line.percentile.width = par("lwd"),
               line.extend.sd = NA, line.extend.se = NA, mean.pch = NA, mean.cex = par("cex"),
               mean.col = par("col"), line.extend.col = par("col"), line.extend.width = par("lwd"),
               point.pattern = "on-line", point.type = "p", point.cex = .7, point.pch = par("pch"),
               point.col = par("col"), xlim = NA, axes = T, xaxt = "s", xlab = "", ylab = "",
               breaks = NA, seed = 500, orientation = "horizontal", vertical.xstart = "left",
               violin.bp = F, violin.sp = F, violin.bp.region = c(.45, .55)) {

  par(cex = text.cex)

  # Extract names of vectors for labeling y-axis. -----------------------------
  # It returns "ylabel"(which is a vector) and "x"(which is a list)
  x <- list(...)
  if (!is.list(x[[1]])) {                       # User entered blip(x1,x2)
    if (all(is.na(ylab))) {
      if (is.null(names(match.call()))) {       # User didn't enter optional arguments
        ylab <- as.character(match.call())
      } else {                                  # User entered optional arguments
        temp <- match.call()
        ylab <- paste(temp[names(temp) == ""])
      }
    }
  } else if (is.null(names(x[[1]]))) {          # User entered blip(list(x1,x2))
      if (all(is.na(ylab))) {
        ylab <- vector()
        for (i in 1:length(x[[1]])) {
          ylab[i] <- i
        }
      }

      temp <- list()
      for (i in 1:length(x[[1]])) {
        temp[[i]] <- x[[1]][[i]]
      }
      x <- temp
  } else {                                      # User entered 'split(x1,x2)'
      x <- x[[1]]
      if (all(is.na(ylab))) {
        ylab <- names(x)
      }
  }

  yPoints <- vector()
  if (std.plot == "violin") {
    original.x <- x
    dens <- density(as.numeric(unlist(x)))
    #print(dens)
    xPoints <- dens$x
    yPoints <- dens$y
    x <- list(as.vector(t(xPoints)))

    graph.type <- "c"
    graph.width <- "v"
    point.type <- "l"
    point.pattern <- "max-range"
  }

  minmax <- check.common.input(x, std.plot, graph.region, nclass, xlim, breaks)

  # Define common variables ---------------------------------------------------
  def <- define.var(x, graph.region)
  x   <- def$x
  n   <- def$n
  low <- def$low
  up  <- def$up
  x.min <- minmax$x.min
  x.max <- minmax$x.max
  nvectors   <- def$nvectors
  boxwidth   <- def$boxwidth
  boxcenter  <- def$boxcenter
  lower.edge <- def$lower.edge
  upper.edge <- def$upper.edge

  #print(x)

  if (std.plot == "boxplot") {
    bp(x, nvectors, lower.edge, upper.edge, xlim, line.extend.col, line.extend.width,
       box.density, box.angle, box.col, point.cex, point.pch, point.col, axes, xaxt, xlab)
  } else if (std.plot == "histogram") {
    histo(x, nvectors, graph.uniform, low, boxwidth, nclass, box.label.cex,
          box.label.above, box.label.on, xlim, line.extend.col, line.extend.width,
          box.density, box.angle, box.col, axes, xaxt, xlab, breaks, x.min, x.max)
  } else if (std.plot == "polygon") {
    polygn(x, nvectors, graph.uniform, low, boxwidth, nclass, box.label.cex,
           box.label.above, box.label.on, xlim, line.extend.col, line.extend.width,
           box.density, box.angle, box.col, axes, xaxt, xlab, breaks, x.min, x.max)
  } else {
    ans1 <- check.input.blip(x, nvectors, box.p, box.p.bar, graph.type, graph.width,
                             point.pattern, line.extend.p, line.extend.sd, line.extend.se)

    xpoints <- ans1$xpoints
    xpoints <- frame.x(x, nvectors, n, mean.pch, mean.cex, mean.col, line.percentile,
                       line.percentile.length, line.percentile.col, line.percentile.width,
                       line.extend.sd, line.extend.se, xlim, graph.type, boxcenter,
                       low, xpoints, line.extend.width, line.extend.col, axes, xaxt,
                       xlab, orientation, vertical.xstart, nclass, boxwidth, graph.uniform)
    box.bounds <- ans1$box.bounds

    ans2 <- draw.box(x, box.p, box.p.bar, box.bounds, nvectors, n, graph.width, graph.type,
                     graph.uniform, point.pattern, nclass, boxcenter, boxwidth, low, up,
                     box.label.on, box.label.cex, box.label.above, box.density, box.angle,
                     box.col, line.extend.width, line.extend.col, orientation)

    draw.line(x, nvectors, line.extend.p, graph.type, boxcenter, low, line.extend.width,
              line.extend.col, orientation)
    draw.point(xpoints, nvectors, n, graph.width, graph.type, graph.uniform, point.pattern,
               ans2, boxwidth, boxcenter, low, up, seed, line.extend.width, line.extend.col,
               point.type, point.cex, point.pch, point.col, orientation, std.plot, yPoints)
  }

  if (std.plot == "violin") {
    par(new = T)
    if (violin.bp)
      bp(original.x, nvectors, violin.bp.region[1], violin.bp.region[2], xlim, line.extend.col, line.extend.width,
         box.density, box.angle, box.col, point.cex, point.pch, point.col, axes = F, xaxt, xlab)
    par(new = T)
    if (violin.sp)
      blip(original.x,box.p=NA,graph.type="c",graph.width="v",point.pattern="jittered",seed=123, axes = F)
  }

  side <- if (orientation == "vertical") 1 else 2
  axis(side, at = boxcenter, tick = FALSE, labels = ylab, adj = 1)
}
