#' Draws the frame of the graph, including the axes, the sd, the se, and the mean.
#'
#' @param x Sorted list of x-values in the dataset.
#' @param nvectors A numeric value indicating the number of groupings in the dataset.
#' @param n A numeric vector indicating the size of each grouping.
#' @param mean.pch A numeric value to specify the symbol for marking the location of the mean.
#' mean.pch=4 corresponds to "x".
#' @param mean.cex A numeric value to specify the size of mean symbol.
#' @param mean.col A numeric value to specify the color of mean symbol.
#' @param line.percentile A numeric vector of percentiles ranging from 0 to 1, indicating segments of lines to be drawn outside the boxes.
#' @param line.percentile.length A numeric value indicating the length of the line.
#' @param line.percentile.col A numeric value indicating the color of the line.
#' @param line.percentile.width A numeric value indicating the width of the line.
#' @param line.extend.sd A numeric value indicating the length of lines to be drawn from the mean in standard deviation.
#' @param line.extend.se A numeric value indicating the length of lines to be drawn from the mean in standard error.
#' @param xlim A numeric vector consists of 2 elements which indicates the range of x-axis.
#' @param graph.type A character value indicating the reference line.
#' "c" for centered and "b" for baseline.
#' @param boxcenter A numerical vector indicating the center of each box.
#' @param low A numerical vector indicating the lower edge of each box.
#' @param xpoints A numerical vector of x points to plot.
#' @param line.extend.width A numeric value to indicate the thickness of the line.
#' @param line.extend.col A numeric value to indicate the color of the line.
#' @param axes A logical value to indicate whether or not to plot axes.
#' @param xaxt "s"=standard x-axis plotting,"n"=null x-axis plotting.
#' @param xlab Labels on x-axis.
#' @param orientation Orientation of the graph. Options are "horizontal" and "vertical".
#' @param vertical.xstart The side the x-values start from when the orientation of the graph is vertical.
#' "left" displays x-values in ascending order; "right" displays x-values in descending order.
#' @param nclass The smoothing parameter which corresponds to how many classes the total number of observations will be divided.
#' @param boxwidth A numerical vector indicating the width of each box.
#' @param graph.uniform A logical value(T/F) to indicate the choice of uniform/non-uniform height on y-axis across groups.
frame.x <- function(x, nvectors, n, mean.pch, mean.cex, mean.col, line.percentile,
                    line.percentile.length, line.percentile.col, line.percentile.width,
                    line.extend.sd, line.extend.se, xlim, graph.type, boxcenter,
                    low, xpoints, line.extend.width, line.extend.col, axes, xaxt,
                    xlab, orientation, vertical.xstart, nclass, boxwidth, graph.uniform) {

  # Calculate sd, se and mean -------------------------------------------------
  mean.x <- vector()
  for (k in 1:nvectors) {
    n[k] <- length(x[[k]])
    mean.x[k] <- mean(x[[k]])
  }

  if (!is.na(line.extend.se)) {
    se.x <- vector()
    qline.extend.se <- list()
    for (k in 1:nvectors) {
      se.x[k] <- sqrt(sum((x[[k]] - mean.x[k])^2) / (n[k] * (n[k] - 1)))
      qline.extend.se[[k]] <- c(mean.x[k] - line.extend.se * se.x[k],
                                mean.x[k] + line.extend.se * se.x[k])
      xpoints[[k]][(xpoints[[k]] >= qline.extend.se[[k]][1]
                & xpoints[[k]] <= qline.extend.se[[k]][2])] <- NA
      xpoints[[k]] <- xpoints[[k]][!is.na(xpoints[[k]])]
    }
  }

  if (!is.na(line.extend.sd)) {
    sd.x <- vector()
    qline.extend.sd <- list()
    for (k in 1:nvectors) {
        sd.x[k] <- sqrt(var(x[[k]]))
        qline.extend.sd[[k]] <- c(mean.x[k] - line.extend.sd * sd.x[k],
                                  mean.x[k] + line.extend.sd * sd.x[k])
        xpoints[[k]][(xpoints[[k]] >= qline.extend.sd[[k]][1]
                      & xpoints[[k]] <= qline.extend.sd[[k]][2])] <- NA
        xpoints[[k]] <- xpoints[[k]][!is.na(xpoints[[k]])]
    }
  }

  if (!is.na(line.percentile[1])) {
    xunique <- list()
    range.p.xunique <- list()
    p.x <- list()
    p.max <- vector()
    delta <- vector()
    length.calc <- list()
    if (graph.uniform) {
        x.max <- max(sapply(x,max))
        x.min <- min(sapply(x,min))
    }

    for (k in 1:nvectors) {
      change <- if (graph.uniform) x.max - x.min else max(x[[k]]) - min(x[[k]])
      delta[k] <- change / (2 * nclass)

      xunique[[k]] <- unique(x[[k]])
      xunique.l <- xunique[[k]] - delta[k]
      xunique.u <- xunique[[k]] + delta[k]
      ranks.x <- sort.list(x[[k]])

      for (i in unique(x[[k]][duplicated(x[[k]])])) {
        ranks.x[x[[k]] == i] <- max(ranks.x[x[[k]] == i])
      }

      p.x[[k]] <- ranks.x / n[k]
      p.xunique.l <- approx(x[[k]], p.x[[k]], xout = xunique.l)$y
      p.xunique.u <- approx(x[[k]], p.x[[k]], xout = xunique.u)$y
      p.xunique.l[is.na(p.xunique.l)] <- 0
      p.xunique.u[is.na(p.xunique.u)] <- 1

      range.p.xunique[[k]] <- p.xunique.u - p.xunique.l
      length.calc[[k]] <- approx(xunique[[k]], range.p.xunique[[k]],  xout = quantile(x[[k]], line.percentile))$y
      p.max[k] <- max(range.p.xunique[[k]])
    }
    linep <- matrix(rep(0, nvectors * length(line.percentile)), length(line.percentile), nvectors)
    for (k in 1:nvectors) {
      linep[, k] <- quantile(x[[k]], line.percentile)
    }
  }

  # Build up basic frame ------------------------------------------------------
  if (!all(is.na(xlim))) {
    fake.x<-xlim
  }
  else {
    x.min <- min(sapply(x, min))
    x.max <- max(sapply(x, max))
    fake.x <- maxmin.frame(x.min, x.max)
    fake3.x <- maxmin.frame(x.min, x.max)

    if (!is.na(line.extend.se) && !is.na(line.extend.sd)) {
      fake1.x <- c(min(sapply(qline.extend.sd, min)), max(sapply(qline.extend.sd, max)))
      fake2.x <- c(min(sapply(qline.extend.se, min)), max(sapply(qline.extend.se, max)))
      fake.x <- c(min(fake1.x[1], fake2.x[1], fake3.x[1]),
                   max(fake1.x[2], fake2.x[2], fake3.x[2]))
    }
    else if (!is.na(line.extend.se)) {
      fake2.x <- c(min(sapply(qline.extend.se, min)), max(sapply(qline.extend.se, max)))
      fake.x <- c(min(fake2.x[1], fake3.x[1]),
                  max(fake2.x[2], fake3.x[2]))
    }
    else if (!is.na(line.extend.sd)) {
      fake1.x <- c(min(sapply(qline.extend.sd, min)), max(sapply(qline.extend.sd, max)))
      fake.x <- c(min(fake1.x[1], fake3.x[1]),
                  max(fake1.x[2], fake3.x[2]))
    }
  }

  # Draw axes -----------------------------------------------------------------
  if (orientation == "vertical")
  {
    if (vertical.xstart == "right") {
      plot(c(0, 1), fake.x, type = "n", axes = F, ylab = xlab, xlab = "", xlim = rev(range(c(0, 1))))
    }
    else {
      plot(c(0, 1), fake.x, type = "n", axes = F, ylab = xlab, xlab = "")
    }

    if (axes) {
      axis(2, yaxt = xaxt)
      box()
    }
  } else {
    plot(fake.x, c(0, 1), type = "n", axes = F, ylab = "", xlab = xlab)
    if (axes) {
      axis(1, xaxt = xaxt)
      box()
    }
  }


  # Draw sd,se,mean -----------------------------------------------------------
  type1.vert.l <- if (orientation != "vertical" || graph.type != "c") "n" else "l"
  type2.vert.l <- if (orientation != "vertical" || graph.type == "c") "n" else "l"
  type1.vert.p <- if (orientation != "vertical" || graph.type != "c") "n" else "p"
  type2.vert.p <- if (orientation != "vertical" || graph.type == "c") "n" else "p"

  type1.horz.l <- if (orientation == "vertical" || graph.type != "c") "n" else "l"
  type2.horz.l <- if (orientation == "vertical" || graph.type == "c") "n" else "l"
  type1.horz.p <- if (orientation == "vertical" || graph.type != "c") "n" else "p"
  type2.horz.p <- if (orientation == "vertical" || graph.type == "c") "n" else "p"

    for (k in 1:nvectors) {
      if (!is.na(line.extend.sd)) {
        lines(rep(boxcenter[k], 2), qline.extend.sd[[k]],
              lwd = line.extend.width, col = line.extend.col, type = type1.vert.l)
        lines(rep(low[k], 2),qline.extend.sd[[k]],
              lwd = line.extend.width, col = line.extend.col, type = type2.vert.l)

        lines(qline.extend.sd[[k]], rep(boxcenter[k], 2),
              lwd = line.extend.width, col = line.extend.col, type = type1.horz.l)
        lines(qline.extend.sd[[k]], rep(low[k], 2),
              lwd = line.extend.width, col = line.extend.col, type = type2.horz.l)
      }

      if (!is.na(line.extend.se)) {
        lines(rep(boxcenter[k], 2), qline.extend.se[[k]],
              lwd = line.extend.width, col = line.extend.col, type = type1.vert.l)
        lines(rep(low[k], 2), qline.extend.se[[k]],
              lwd = line.extend.width, col = line.extend.col, type = type2.vert.l)

        lines(qline.extend.se[[k]], rep(boxcenter[k], 2),
              lwd = line.extend.width, col = line.extend.col, type = type1.horz.l)
        lines(qline.extend.se[[k]], rep(low[k], 2),
              lwd = line.extend.width, col = line.extend.col, type = type2.horz.l)
      }

      if (!is.na(mean.pch)) {
        points(boxcenter[k], mean.x[k], pch = mean.pch, col = mean.col, cex = mean.cex, type = type1.vert.p)
        points(low[k], mean.x[k], pch = mean.pch, col = mean.col, cex = mean.cex, type = type2.vert.p)

        points(mean.x[k], boxcenter[k], pch = mean.pch, col = mean.col, cex = mean.cex, type = type1.horz.p)
        points(mean.x[k], low[k], pch = mean.pch,col = mean.col, cex = mean.cex, type = type2.horz.p)
      }

      if (!is.na(line.percentile[1])) {
        for (l in 1:length(line.percentile))
        {
          real.length <-
            if (is.na(line.percentile.length))
              length.calc[[k]][l] / 2 / p.max[k] * (n[k] / max(n)) * boxwidth[k]
          else
            line.percentile.length / 2

          lines(c(boxcenter[k] - real.length, boxcenter[k] + real.length),
                c(linep[l, k], linep[l, k]), lwd = line.percentile.width,
                col = line.percentile.col, type = type1.vert.l)
          lines(c(low[k], low[k] + real.length * 2),
                c(linep[l, k], linep[l, k]), lwd = line.percentile.width,
                col = line.percentile.col, type = type2.vert.l)

          lines(c(linep[l, k], linep[l, k]),
                c(boxcenter[k] - real.length, boxcenter[k] + real.length),
                lwd = line.percentile.width, col = line.percentile.col, type = type1.horz.l)
          lines(c(linep[l, k], linep[l, k]),
                c(low[k], low[k] + real.length * 2), lwd = line.percentile.width,
                col = line.percentile.col, type = type2.horz.l)
        }
      }
    }
  return(xpoints)
}


#' Draws boxs on the graph.
#'
#' @param x Sorted list of x-values in the dataset.
#' @param box.p A numeric vector of percentiles ranging from 0 to 1.
#' Missing values are not allowed in the vector except box.p=NA.
#' Numeric value 0 can be used as a separator for separatingboxes.
#' Each box must contain at least two percentiles.
#' @param box.p.bar A vector of logical value indicating whether to draw the vertical bars on percentiles.
#' @param box.bounds A numeric vector containing the boundaries of each box.
#' @param nvectors A numeric value indicating the number of groupings in the dataset.
#' @param n A numeric vector indicating the size of each grouping.
#' @param graph.width A character value indicating the type of width of the box.
#' "f" for fixed and "v" for variable.
#' @param graph.type A character value indicating the reference line.
#' "c" for centered and "b" for baseline.
#' @param graph.uniform A logical value(T/F) to indicate the choice of uniform/non-uniform height on y-axis across groups.
#' @param point.pattern When point.pattern="on-line", the points outside the boxes will be displayed on the reference line.
#' When point.pattern="stacking", the points outside the boxes will be separated by a fixed distance if there are more than one points with the same value.
#' When point.pattern="evenly-spaced", the points outside the boxes will be evenly spreaded out if there are more than one points with the same value.
#' When point.pattern="jittered", the points outside the boxes will be displayed randomly within the graph width.
#' When point.pattern="max-range", the points outside the boxes will be drawn at the maximum height of that point.
#' When point.pattern="vertical-bar", a vertical bar instead of a point will be drawn at the location of each observation.
#' @param nclass The smoothing parameter which corresponds to how many classes the total number of observations will be divided.
#' @param boxcenter A numerical vector indicating the center of each box.
#' @param boxwidth A numerical vector indicating the width of each box.
#' @param low A numerical vector indicating the lower edge of each box.
#' @param up A numerical vector indicating the higher edge of each box.
#' @param box.label.on A Logical value indicating whether to label the percentiles on the boxes.
#' @param box.label.cex A numeric value to control the size of the labels.
#' @param box.label.above A numeric value indicating the distance of the labels above the box.
#' @param box.density A numeric value to indicate the density of the box shading lines.
#' @param box.angle A numeric value to indicate the angle of the box shading lines.
#' @param box.col A vector to indicate the color(s) of the box shading lines.
#' @param line.extend.width A numeric value to indicate the thickness of the line.
#' @param line.extend.col A numeric value to indicate the color of the line.
#' @param orientation Orientation of the graph. Options are "horizontal" and "vertical".
#'
#' @return A list containing xunique, range.p.xunique, p.max
draw.box <- function(x, box.p, box.p.bar, box.bounds, nvectors, n,
                   graph.width, graph.type, graph.uniform, point.pattern,
                   nclass, boxcenter, boxwidth, low, up, box.label.on,
                   box.label.cex, box.label.above, box.density, box.angle,
                   box.col, line.extend.width, line.extend.col, orientation) {

  # Determine the range of percentile at each point for drawing of box plot ---
  nboxs <- length(box.p)
  xunique <- list()
  range.p.xunique <- list()
  p.x <- list()
  p.max <- vector()
  delta <- vector()

  if (graph.uniform) {
    x.max <- max(sapply(x, max))
    x.min <- min(sapply(x, min))
  }

  if (graph.width=="v") {
    for (k in 1:nvectors)
    {
      delta[k] <-
        if (graph.uniform) (x.max - x.min) / (2 * nclass)
        else   (max(x[[k]]) - min(x[[k]])) / (2 * nclass)
      xunique[[k]] <- unique(x[[k]])
      xunique.l <- xunique[[k]] - delta[k]
      xunique.u <- xunique[[k]] + delta[k]
      ranks.x <- sort.list(x[[k]])
      for (i in unique(x[[k]][duplicated(x[[k]])])) {
        ranks.x[x[[k]] == i] <- max(ranks.x[x[[k]] == i])
      }

      p.x[[k]] <- ranks.x / n[k]
      p.xunique.l <- approx(x[[k]], p.x[[k]], xout = xunique.l)$y
      p.xunique.u <- approx(x[[k]], p.x[[k]], xout = xunique.u)$y
      p.xunique.l[is.na(p.xunique.l)] <- 0
      p.xunique.u[is.na(p.xunique.u)] <- 1
      range.p.xunique[[k]] <- p.xunique.u - p.xunique.l
      p.max[k] <- max(range.p.xunique[[k]])
    }

    if (graph.uniform) {
      p.max <- rep(max(p.max), length(p.max))
    }
  }

  # Determine the height of the box -------------------------------------------
  if (all(!is.na(box.p))) {
    for (k in 1:nvectors) {
      boxcolor <-
        if (k %% length(box.col)) box.col[k %% length(box.col)]
        else box.col[length(box.col)]

      q  <- quantile(x[[k]], probs = box.p)
      nq <- length(q)

      if (graph.width == "f") {                       # Fixed width box
        loweredge <- rep(low[k], nq)
        upperedge <- rep(up[k],  nq)
      } else {                                        # Variable width box
        q.l <- q - delta[k]
        q.u <- q + delta[k]
        p.q.l <- approx(x[[k]], p.x[[k]], xout = q.l)$y
        p.q.u <- approx(x[[k]], p.x[[k]], xout = q.u)$y
        p.q.l[is.na(p.q.l)] <- 0
        p.q.u[is.na(p.q.u)] <- 1
        range.q <- p.q.u - p.q.l
        range.q[range.q > p.max[k]] <- p.max[k]

        loweredge <-
          if (graph.type != "c")
            rep(low[k], nq)
          else if (graph.uniform)
            boxcenter[k] - ((range.q / p.max[k]) * (n[k] / max(n)) * boxwidth[k]) / 2
          else
            boxcenter[k] - ((range.q / p.max[k]) * boxwidth[k]) / 2
        upperedge <-
          if (graph.type != "c" && graph.uniform)
              low[k] + (range.q / p.max[k]) * (n[k] / max(n)) * boxwidth[k]
          else if (graph.type != "c" && !graph.uniform)
            low[k] + (range.q / p.max[k]) * boxwidth[k]
          else if (graph.uniform)
            upperedge <- boxcenter[k] + ((range.q / p.max[k]) * (n[k] / max(n)) * boxwidth[k]) / 2
          else
            upperedge <- boxcenter[k] + ((range.q / p.max[k]) * boxwidth[k]) / 2

      }

      # Draw the box ----------------------------------------------------------
      for (i in 1:(dim(box.bounds)[1])) {
        q.box.bounds <- quantile(x[[k]], probs = box.bounds[i, ])
        q.box.bounds <- q[q >= q.box.bounds[1] & q <= q.box.bounds[2]]
        hi <- upperedge[box.p >= box.bounds[i, 1] & box.p <= box.bounds[i, 2]]
        lo <- loweredge[box.p >= box.bounds[i, 1] & box.p <= box.bounds[i, 2]]

        v1 <- c(hi,rev(lo))
        v2 <- c(q.box.bounds,rev(q.box.bounds))
        if (orientation == "vertical") {
          polygon(v1, v2, border = F, density = box.density, angle = box.angle,
                  col = boxcolor, lwd = line.extend.width)

        } else {
          polygon(v2, v1, border = F, density = box.density, angle = box.angle,
                  col = boxcolor, lwd = line.extend.width)
        }
      }

      q[(2:nboxs)[box.p[2:nboxs] == 0]] <- NA
      qnames <- names(q)
      if (any(box.p.bar)) {
        if (orientation == "vertical") {
          segments(loweredge[box.p.bar == T], q[box.p.bar == T],
                   upperedge[box.p.bar == T], q[box.p.bar == T],
                   lwd = line.extend.width, col = line.extend.col)

        } else {
          segments(q[box.p.bar == T], loweredge[box.p.bar == T],
                   q[box.p.bar == T], upperedge[box.p.bar == T],
                   lwd = line.extend.width, col = line.extend.col)
        }
      }

      if (orientation=="vertical") {
        lines(loweredge, q, lwd = line.extend.width, col = line.extend.col)
        lines(upperedge, q, lwd = line.extend.width, col = line.extend.col)
      } else {
        lines(q, loweredge, lwd = line.extend.width, col = line.extend.col)
        lines(q, upperedge, lwd = line.extend.width, col = line.extend.col)
      }

      if (box.label.on) {
        text(q[box.p.bar == T], upperedge[box.p.bar == T] + box.label.above,
             qnames[box.p.bar == T], cex = box.label.cex)
      }
    }
  }
  return(list(xunique = xunique, range.p.xunique = range.p.xunique, p.max = p.max))
}


#' Draws lines on the graph.
#'
#' @param x Sorted list of x-values in the dataset.
#' @param nvectors A numeric value indicating the number of groupings in the dataset.
#' @param line.extend.p A numeric vector of percentiles ranging from 0 to 1, indicating segments of lines to be drawn outside the boxes.
#' @param graph.type A character value indicating the reference line.
#' "c" for centered and "b" for baseline.
#' @param boxcenter A numerical vector indicating the center of each box.
#' @param low A numerical vector indicating the lower edge of each box.
#' @param line.extend.width A numeric value to indicate the thickness of the line.
#' @param line.extend.col A numeric value to indicate the color of the line.
#' @param orientation Orientation of the graph. Options are "horizontal" and "vertical".
draw.line <- function(x, nvectors, line.extend.p, graph.type, boxcenter, low,
                    line.extend.width, line.extend.col, orientation) {

  # Draw lines in line.extend.p option ----------------------------------------
  if (any(!is.na(line.extend.p))) {
    qlines <- list()
    nlines <- length(line.extend.p)
    for (k in 1:nvectors) {
      qlines[[k]] <- quantile(x[[k]], probs = line.extend.p)
      nqlines <- length(qlines[[k]])
      qlines[[k]][(2:nlines)[line.extend.p[2:nlines] == 0]] <- NA

      type1.vert <- if (orientation == "vertical" && graph.type == "c") "l" else "n"
      type2.vert <- if (orientation == "vertical" && graph.type != "c") "l" else "n"
      type1.horz <- if (orientation != "vertical" && graph.type == "c") "l" else "n"
      type2.horz <- if (orientation != "vertical" && graph.type != "c") "l" else "n"

      lines(rep(boxcenter[k], nqlines), qlines[[k]], type = type1.vert,
            lwd = line.extend.width, col = line.extend.col)
      lines(rep(low[k], nqlines), qlines[[k]], type = type2.vert,
            lwd = line.extend.width, col = line.extend.col)
      lines(qlines[[k]], rep(boxcenter[k], nqlines), type = type1.horz,
            lwd = line.extend.width, col = line.extend.col)
      lines(qlines[[k]], rep(low[k], nqlines), type = type2.horz,
            lwd = line.extend.width, col = line.extend.col)
    }
  }
}


#' Draws points on the graph.
#'
#' @param xpoints A numerical vector of x points to plot.
#' @param nvectors A numeric value indicating the number of groupings in the dataset.
#' @param n A numeric vector indicating the size of each grouping.
#' @param graph.width A character value indicating the type of width of the box.
#' "f" for fixed and "v" for variable.
#' @param graph.type A character value indicating the reference line.
#' "c" for centered and "b" for baseline.
#' @param graph.uniform A logical value(T/F) to indicate the choice of uniform/non-uniform height on y-axis across groups.
#' @param point.pattern When point.pattern="on-line", the points outside the boxes will be displayed on the reference line.
#' When point.pattern="stacking", the points outside the boxes will be separated by a fixed distance if there are more than one points with the same value.
#' When point.pattern="evenly-spaced", the points outside the boxes will be evenly spreaded out if there are more than one points with the same value.
#' When point.pattern="jittered", the points outside the boxes will be displayed randomly within the graph width.
#' When point.pattern="max-range", the points outside the boxes will be drawn at the maximum height of that point.
#' When point.pattern="vertical-bar", a vertical bar instead of a point will be drawn at the location of each observation.
#' @param ans2 A numerical vector returned by draw.box containing information for variable with graphs.
#' @param boxwidth A numerical vector indicating the width of each box.
#' @param boxcenter A numerical vector indicating the center of each box.
#' @param low A numerical vector indicating the lower edge of each box.
#' @param up A numerical vector indicating the higher edge of each box.
#' @param seed An integer. This is a seed for the random number generator which is used in point.pattern="jittered".
#' @param line.extend.width A numeric value to indicate the thickness of the line.
#' @param line.extend.col A numeric value to indicate the color of the line.
#' @param point.type A character value to indicate whether a point("p"),a line("l"), both("b"), or none("n") will be drawn
#' @param point.cex A numeric value to specify the size of points
#' @param point.pch A numeric or character value to specify the shape of points.
#' @param point.col A numeric value to specify the color of the points.
#' @param orientation Orientation of the graph. Options are "horizontal" and "vertical".
#' @param std.plot This option overrides all other related graphical parameters.
#' std.plot="boxplot" draws conventional boxplots;
#' std.plot="histogram" produces histogram;
#' std.plot="polygon" plots frequency polygon.
#' @param yPoints Optional parameter specifically for violin plots, which directly alters the density estimates.
draw.point<-function(xpoints, nvectors, n, graph.width, graph.type, graph.uniform,
                     point.pattern, ans2, boxwidth, boxcenter, low, up,
                     seed, line.extend.width, line.extend.col, point.type, point.cex,
                     point.pch, point.col, orientation, std.plot, yPoints) {

  # Determine the value of y-axis of points that are to be drawn --------------
  if (any(!is.na(xpoints))) {
    if (point.pattern == "stacking") {
      tie.matrix <- list()
      max.tie <- vector()
      for (k in 1:nvectors) {
        xpoints.unique <- unique(xpoints[[k]])
        tie.matrix[[k]] <- matrix(NA,length(xpoints.unique), 2)
        for (i in (1:length(xpoints.unique))) {
          tie.matrix[[k]][i,]<- c(xpoints.unique[i],
                                  sum(xpoints[[k]] == xpoints.unique[i]))
        }
        max.tie[k] <- max(tie.matrix[[k]][, 2])
      }
      increment <-
        if (graph.uniform)
          boxwidth / max(max.tie)
        else
          boxwidth / max.tie
    }

    range.p.xunique <- ans2$range.p.xunique
    p.max <- ans2$p.max
    xunique <- ans2$xunique
    for (k in 1:nvectors) {
      nxpoints <- length(xpoints[[k]])
      if (point.pattern == "on-line") {
        ypoints <-
          if (graph.type == "b")
            rep(low[k], nxpoints)
          else
            rep(boxcenter[k], nxpoints)
      } else if (point.pattern == "jittered") {
        ypoints <- vector()
        set.seed(seed)
        if (graph.width == "f") {
          ypoints<-runif(nxpoints,low[k],up[k])
        } else {
          height.xpoints<-vector()
          for (i in 1:nxpoints) {
            height.xpoints[i] <-
              if (graph.uniform)
                (range.p.xunique[[k]][xunique[[k]] == xpoints[[k]][i]]
                 / p.max[k]) * (n[k] / max(n)) * boxwidth[k]
              else
                (range.p.xunique[[k]][xunique[[k]] == xpoints[[k]][i]]
                 / p.max[k]) * boxwidth[k]

            ypoints[i] <-
              if (graph.type == "b")
                runif(1, low[k], (height.xpoints[i] + low[k]))
              else
                runif(1, (boxcenter[k] - height.xpoints[i] / 2),
                      (boxcenter[k] + height.xpoints[i] / 2))
          }
          #print(height.xpoints)
        }
      } else if (point.pattern == "max-range") {
        ypoints <- vector()
        ypoints2 <- vector()
        height.xpoints <- vector()
        for (i in 1:nxpoints) {
          if (graph.width == "f") {
            ypoints[i] <- up[k]
          } else {
            height.xpoints[i] <-
              if (graph.uniform)
                (range.p.xunique[[k]][xunique[[k]] == xpoints[[k]][i]]
                 / p.max[k]) * (n[k] / max(n)) * boxwidth[k]
              else
                (range.p.xunique[[k]][xunique[[k]] == xpoints[[k]][i]]
                 / p.max[k]) * boxwidth[k]

            if (std.plot == "violin")
              height.xpoints[i] <- yPoints[i]*100

            ypoints[i] <-
              if (graph.type == "b")
                low[k] + height.xpoints[i]
              else
                boxcenter[k] + height.xpoints[i] / 2
            ypoints2[i] <-
              if (graph.type == "b")
                low[k] - height.xpoints[i]
              else
                boxcenter[k] - height.xpoints[i] / 2
          }
        }

      } else if (point.pattern == "vertical-bar") {
        ypoints.up  <- vector()
        ypoints.low <- vector()
        height.xpoints <- vector()
        for (i in 1:nxpoints) {
          if (graph.width == "f") {
            ypoints.up[i]  <- up[k]
            ypoints.low[i] <- low[k]
          } else {
            height.xpoints[i] <-
              if (graph.uniform)
                (range.p.xunique[[k]][xunique[[k]] == xpoints[[k]][i]] / p.max[k]) * (n[k] / max(n)) * boxwidth[k]
              else
                (range.p.xunique[[k]][xunique[[k]] == xpoints[[k]][i]] / p.max[k]) * boxwidth[k]

            ypoints.up[i] <-
              if (graph.type == "b")
                low[k] + height.xpoints[i]
              else
                boxcenter[k] + height.xpoints[i] / 2
            ypoints.low[i] <-
              if (graph.type == "b")
                low[k]
              else
                boxcenter[k] - height.xpoints[i] / 2
          }
        }
      } else if (point.pattern == "evenly-spaced") {
        ypoints <- vector()
        xpoints.unique <- unique(xpoints[[k]])
        tie.matrix <- matrix(NA, length(xpoints.unique), 2)
        if (graph.width == "v") {
          height.xpoints <- vector()
        }
        j <- 1
        for (i in (1:length(xpoints.unique))) {
          tie.matrix[i,] <- c(xpoints.unique[i],
                              sum(xpoints[[k]] == xpoints.unique[i]))
          if (graph.width == "v") {
            height.xpoints[i] <-
              if (graph.uniform)
                (range.p.xunique[[k]][xunique[[k]] == tie.matrix[i, 1]] / p.max[k]) * (n[k] / max(n)) * boxwidth[k]
              else
                (range.p.xunique[[k]][xunique[[k]] == tie.matrix[i, 1]] / p.max[k]) * boxwidth[k]
          }

          if (tie.matrix[i,2]==1) {
            ypoints[j] <-
              if (graph.type == "c")
                boxcenter[k]
              else if (graph.width == "f")
                low[k]
              else
                low[k] + height.xpoints[i] / 2
            j <- j+1
          } else if (tie.matrix[i,2] > 1) {
            for (l in (0:(tie.matrix[i,2]-1)))
            {
              ypoints[j] <-
                if (graph.width == "f")
                  low[k] + (boxwidth[k] / (tie.matrix[i,2] - 1)) * l
                else if (graph.type == "c")
                  (boxcenter[k] - height.xpoints[i] / 2 +
                     (height.xpoints[i] / (tie.matrix[i, 2] - 1)) * l)
                else
                  low[k] + (height.xpoints[i] / (tie.matrix[i, 2] - 1)) * l
              j <- j+1
            }
          }
        }
      } else {
        ypoints <- vector()
        xpoints.unique <- unique(xpoints[[k]])
        j <- 1
        for (i in (1:length(xpoints.unique))) {
          if (tie.matrix[[k]][i,2] == 1) {
            ypoints[j] <-
              if (graph.type == "b")
                low[k]
              else
                boxcenter[k]
            j <- j+1
          }

           if (tie.matrix[[k]][i, 2] > 1) {
            for (l in (1:tie.matrix[[k]][i, 2])) {
              ypoints[j] <-
                if (graph.type == "b")
                  low[k] + increment[k] * (l - 1)
                else
                  boxcenter[k] - increment[k] * (tie.matrix[[k]][i, 2] - 1) / 2 + increment[k] * (l - 1)
              j <- j+1
            }
          }
        }
      }

      # Draw the points -------------------------------------------------------
      if (orientation == "vertical") {
        if (point.pattern != "vertical-bar") {
          if (std.plot == "violin") {
            lines(smooth.spline(ypoints, xpoints[[k]], spar = 0.3), col = "red")
            lines(smooth.spline(ypoints2, xpoints[[k]], spar = 0.3), col = "red")
          } else {
            points(ypoints, xpoints[[k]], pch = point.pch, cex = point.cex,
                   type = point.type, col = point.col)
          }
        } else {
          segments(ypoints.up, xpoints[[k]], ypoints.low, xpoints[[k]],
                   lwd = line.extend.width, col = line.extend.col)
        }
      } else {
        if (point.pattern != "vertical-bar") {
          if (std.plot == "violin") {
            lines(smooth.spline(xpoints[[k]], ypoints - .05, spar = 0.35), col = "red")
            lines(smooth.spline(xpoints[[k]], ypoints2 - .05, spar = 0.35), col = "red")
          } else {
            points(xpoints[[k]], ypoints, pch = point.pch, cex = point.cex,
                   type = point.type, col = point.col)
          }
        } else {
          segments(xpoints[[k]], ypoints.up, xpoints[[k]], ypoints.low,
                   lwd = line.extend.width, col = line.extend.col)
        }
      }
    }
  }
}
