#' Checks for input errors.
#'
#' @param x Sorted list of x-values in the dataset.
#' @param std.plot This option overrides all other related graphical parameters.
#' std.plot="boxplot" draws conventional boxplots;
#' std.plot="histogram" produces histogram;
#' std.plot="polygon" plots frequency polygon.
#' @param graph.region A numerical vector with 2 elements which specifies the lower and upper graphical region between 0 and 1.
#' @param nclass The smoothing parameter which corresponds to how many classes the total number of observations will be divided.
#' @param xlim A numeric vector consists of 2 elements which indicates the range of x-axis.
#' @param breaks A numeric vector indicating break points for histogram and polygon.
#'
#' @return A list holding the smallest break point followed by the largest break point
check.common.input <- function(x, std.plot, graph.region, nclass, xlim, breaks) {
  if (std.plot != "boxplot" &  std.plot != "histogram" & std.plot != "polygon" & std.plot != "violin" & std.plot != "")
    stop("Incorrect value for std.plot option")
  if (graph.region[1] >= graph.region[2])
    stop("Value of the lower edge of a box has to be smaller than that of upper edge")
  if (length(graph.region) != 2)
    stop("Incorrect values for graph.region option")
  if (!is.numeric(nclass))
    stop("Non-numeric value for nclass")
  if (any(!is.na(xlim)) & (xlim[2] <= xlim[1] | length(xlim) != 2))
      stop("Incorrect value for xlim")
  x.min <- min(sapply(x, min))
  x.max <- max(sapply(x, max))
  if ((std.plot == "histogram" | std.plot == "polygon") & all(!is.na(breaks))) {
    if (breaks[1] > x.min | breaks[length(breaks)] < x.max)
      stop("Break points should cover the range of the data")
    if (!all(breaks == sort(breaks)))
      stop("Break points are not in ascending order")
    x.min <- breaks[1]
    x.max <- breaks[length(breaks)]
  }
  return(list(x.min = x.min, x.max = x.max))
}


#' Defines common variables
#'
#' @param x Sorted list of x-values in the dataset.
#' @param graph.region A numerical vector with 2 elements which specifies the lower and upper graphical region between 0 and 1.
#'
#' @return A list containing x, nvectors, n, low, up, boxwidth, boxcenter, lower.edge, and upper.edge.
define.var <- function(x, graph.region) {
  x  <- lapply(x, sort)
  n  <- vector()
  up  <- vector()
  low <- vector()
  nvectors <- length(x)
  boxwidth <- vector()
  boxcenter <- vector()
  lower.edge <- graph.region[1]
  upper.edge <- graph.region[2]
  for (k in 1:nvectors) {
    n[k] <- length(x[[k]])
    low[k] <- (k - 1) / nvectors + (1 / nvectors) * lower.edge
    up[k]  <- (k - 1) / nvectors + (1 / nvectors) * upper.edge
    boxwidth[k]  <- up[k] - low[k]
    boxcenter[k] <- low[k] + boxwidth[k] / 2
  }

  return(list(x = x, nvectors = nvectors, n = n, low = low, up = up, boxwidth = boxwidth,
              boxcenter = boxcenter, lower.edge = lower.edge, upper.edge = upper.edge))
}


#' Checks for input errors relating to gui.
#'
#' @param x Sorted list of x-values in the dataset.
#' @param nvectors A numeric value indicating the number of groupings in the dataset.
#' @param box.p A numeric vector of percentiles ranging from 0 to 1.
#' Missing values are not allowed in the vector except box.p=NA.
#' Numeric value 0 can be used as a separator for separatingboxes.
#' Each box must contain at least two percentiles.
#' @param box.p.bar A vector of logical value indicating whether to draw the vertical bars on percentiles.
#' @param graph.type A character value indicating the reference line.
#' "c" for centered and "b" for baseline.
#' @param graph.width A character value indicating the type of width of the box.
#' "f" for fixed and "v" for variable.
#' @param point.pattern When point.pattern="on-line", the points outside the boxes will be displayed on the reference line.
#' When point.pattern="stacking", the points outside the boxes will be separated by a fixed distance if there are more than one points with the same value.
#' When point.pattern="evenly-spaced", the points outside the boxes will be evenly spreaded out if there are more than one points with the same value.
#' When point.pattern="jittered", the points outside the boxes will be displayed randomly within the graph width.
#' When point.pattern="max-range", the points outside the boxes will be drawn at the maximum height of that point.
#' When point.pattern="vertical-bar", a vertical bar instead of a point will be drawn at the location of each observation.
#' @param line.extend.p A numeric vector of percentiles ranging from 0 to 1, indicating segments of lines to be drawn outside the boxes.
#' @param line.extend.sd A numeric value indicating the length of lines to be drawn from the mean in standard deviation.
#' @param line.extend.se A numeric value indicating the length of lines to be drawn from the mean in standard error.
#'
#' @return A list containing xpoints and box.bounds
check.input.blip <- function(x, nvectors, box.p, box.p.bar, graph.type, graph.width,
                             point.pattern, line.extend.p, line.extend.sd, line.extend.se) {
  nboxs <- length(box.p)
  if (length(line.extend.sd) > 1)
    stop("Only one value is needed for line.extend.sd option")
  if (length(line.extend.se) > 1)
    stop("Only one value is needed for line.extend.se option")
  if (graph.type != "b" & graph.type != "c" )
    stop("Incorrect value for graph.type")
  if (graph.width != "f" & graph.width != "v")
    stop("Incorrect value for graph.width")
  if (point.pattern != "on-line" & point.pattern != "stacking" &
      point.pattern != "evenly-spaced" & point.pattern != "jittered" &
      point.pattern != "max-range" & point.pattern != "vertical-bar")
    stop("Incorrect value for point.pattern")

  temp1 <- check.p.option(box.p, nboxs)
  box.bounds <- temp1$box.bounds
  nreal.breakpt <- temp1$nreal.breakpt

  # Check box.p.bar option ----------------------------------------------------
  if (any(is.numeric(box.p.bar)))
    stop("No numeric value is allowed for box.p.bar option")

  if (any(!is.na(box.p)) & length(box.p.bar) != nboxs)
    stop("Incorrect value for box.p.bar option")

  temp2 <- check.line.extend.p.option(box.p, line.extend.p, box.bounds)
  nline.breakpt <- temp2$nline.breakpt
  line.bounds <- temp2$line.bounds

  # Exclude points inside box and line ----------------------------------------
  xpoints <- x
  if (any(!is.na(box.p)) | any(!is.na(line.extend.p))) {
    for (k in 1:nvectors) {
      if (any(!is.na(line.extend.p))) {
        for (i in 1:(nline.breakpt + 1)) {
          xpoints[[k]][((xpoints[[k]] >= quantile(x[[k]],   probs = line.bounds[i, 1])) &
                          (xpoints[[k]] <= quantile(x[[k]], probs = line.bounds[i, 2])))] <- NA
        }
        xpoints[[k]] <- xpoints[[k]][!is.na(xpoints[[k]])]
      }
      if (any(!is.na(box.p))) {
        for (i in 1:(nreal.breakpt + 1)) {
          xpoints[[k]][((xpoints[[k]] >= quantile(x[[k]], probs = box.bounds[i, 1])) &
                          (xpoints[[k]] <= quantile(x[[k]], probs = box.bounds[i, 2])))] <- NA
        }
        xpoints[[k]] <- xpoints[[k]][!is.na(xpoints[[k]])]
      }
    }
  }
  if (all(is.na(box.p))) {
    box.bounds <- NA
  }

  return(list(xpoints = xpoints, box.bounds = box.bounds))
}


#' Checks input errors related to the p options.
#'
#' @param box.p A numeric vector of percentiles ranging from 0 to 1.
#' Missing values are not allowed in the vector except box.p=NA.
#' Numeric value 0 can be used as a separator for separatingboxes.
#' Each box must contain at least two percentiles.
#' @param nboxs The number of boxes to be drawn in the plot.
#'
#' @return The boundaries of each box.
check.p.option <- function(box.p, nboxs) {
  if ((any(is.na(box.p))) & nboxs > 1)
    stop("Incorrect value in box.p option")
  if (all(!is.na(box.p))) {
    if (!all(box.p[box.p != 0] == sort(box.p[box.p != 0])))
      stop("Percentiles are not in an increasing order")
    if (nboxs < 2)
      stop("Minimum two percentiles are needed for each box")
    if (any(duplicated(box.p[box.p != 0])))
      stop("Duplicated percentiles are not allowed")
    if (any(box.p > 1) | any(box.p < 0))
      stop("box.p has to be between 0 and 1")
    if (box.p[nboxs] == 0)
      stop("The last element of box.p cannot be 0")
    if (all(box.p == 0))
      stop("All elements of box.p being 0 is not allowed")

    true.p <- box.p
    if (box.p[1] == 0 & box.p[2] == 0) true.p[1] <- true.p[2] <- NA
    if (box.p[(nboxs - 1)] == 0 & box.p[nboxs] == 1 & nboxs > 2) {
      true.p[(nboxs - 1)] <- true.p[nboxs] <- NA
    }
    true.p <- true.p[!is.na(true.p)]
    real.breakpt  <- (2:length(true.p))[true.p[2:length(true.p)] == 0]
    nreal.breakpt <- length(real.breakpt)
    box.bounds <- matrix(0, (nreal.breakpt + 1), 2)

    box.bounds[1, 1] <- true.p[1]
    box.bounds[1, 2] <-
      if (nreal.breakpt == 0)
        true.p[length(true.p)]
    else
      true.p[real.breakpt[1] - 1]
    if (box.bounds[1, 1] == box.bounds[1, 2])
      stop("At least two values for each box")

    if (nreal.breakpt != 0) {
      box.bounds[nreal.breakpt + 1, 1] <- true.p[real.breakpt[nreal.breakpt] + 1]
      box.bounds[nreal.breakpt + 1, 2] <- true.p[length(true.p)]
      if (box.bounds[nreal.breakpt + 1, 1] == box.bounds[nreal.breakpt + 1, 2])
        stop("At least two values for each box")
      if (nreal.breakpt > 1) {
        for (i in 1:(nreal.breakpt - 1)) {
          box.bounds[i + 1, 1] <- true.p[real.breakpt[i] + 1]
          box.bounds[i + 1, 2] <- true.p[real.breakpt[i + 1] - 1]
          if (box.bounds[i + 1, 1] == box.bounds[i + 1, 2])
            stop("Minimun two values needed for each box")
        }
      }
    }
    return(list(box.bounds = box.bounds, nreal.breakpt = nreal.breakpt))
  }
}


#' Checks input errors related to the line.extend.p options.
#'
#' @param box.p A numeric vector of percentiles ranging from 0 to 1.
#' Missing values are not allowed in the vector except box.p=NA.
#' Numeric value 0 can be used as a separator for separatingboxes.
#' Each box must contain at least two percentiles.
#' @param line.extend.p A numeric vector of percentiles ranging from 0 to 1,
#' indicating segments of lines to be drawn outside the boxes.
#' @param box.bounds The boundaries of each box to be drawn.
#'
#' @return nline.breakpt
check.line.extend.p.option <- function(box.p, line.extend.p, box.bounds) {
  if (any(!is.na(line.extend.p))) {
    nlines <- length(line.extend.p)
    if (!all(line.extend.p[line.extend.p != 0] == sort(line.extend.p[line.extend.p != 0])))
      stop("Percentiles are not in an increasing order")
    if (any(duplicated(line.extend.p[line.extend.p != 0])))
      stop("Duplicated percentiles are not allowed")
    line.breakpt  <- (2:nlines)[line.extend.p[2:nlines] == 0]
    nline.breakpt <- length(line.breakpt)
    line.bounds <- matrix(0, nline.breakpt + 1, 2)
    line.bounds[1, 1] <- line.extend.p[1]
    line.bounds[1, 2] <-
      if (nline.breakpt == 0)
        line.extend.p[nlines]
    else
      line.extend.p[line.breakpt[1] - 1]

    if (line.bounds[1, 1] == line.bounds[1, 2])
      stop("At least two values for each line segment")
    if (nline.breakpt != 0) {
      line.bounds[nline.breakpt + 1, 1] <- line.extend.p[line.breakpt[nline.breakpt] + 1]
      line.bounds[nline.breakpt + 1, 2] <- line.extend.p[nlines]
      if (line.bounds[nline.breakpt + 1, 1] == line.bounds[nline.breakpt + 1, 2])
        stop("At least two values for each line segment")
      if (nline.breakpt > 1) {
        for (j in 1:(nline.breakpt - 1)) {
          line.bounds[j + 1, 1] <- line.extend.p[line.breakpt[j] + 1]
          line.bounds[j + 1, 2] <- line.extend.p[line.breakpt[j + 1] - 1]
          if (line.bounds[j + 1, 1] == line.bounds[j + 1, 2])
            stop("At least two values for each line segment")
        }
      }
    }
    if (all(!is.na(box.p))) {
      for (i in 1:dim(line.bounds)[1]) {
        for (j in 1:dim(box.bounds)[1]) {
          if ((line.bounds[i, 1] > box.bounds[j, 1] & line.bounds[i, 2] < box.bounds[j, 2]) |
              (line.bounds[i, 1] < box.bounds[j, 1] & line.bounds[i, 2] > box.bounds[j, 2]) |
              (line.bounds[i, 1] > box.bounds[j, 1] & line.bounds[i, 1] < box.bounds[j, 2]) |
              (line.bounds[i, 2] > box.bounds[j, 1] & line.bounds[i, 2] < box.bounds[j, 2]))
            stop("Incorrect values for line.extend.p option")
        }
      }
    }
    return(list(nline.breakpt = nline.breakpt, line.bounds = line.bounds))
  }
}


#' Computes and returns a minimum and maximum value for the x-axis.
#'
#' @param x.min A numeric value indicating the smallest break point.
#' @param x.max A numeric value indicating the highest break point.
#'
#' @return A vector of the low and high boundaries of the x-axis.
maxmin.frame <- function(x.min, x.max) {
  fake.x.min <-
    if (x.min >= 0)
      .95 * x.min
    else
      1.05 * x.min
  fake.x.max <-
    if (x.max >= 0)
      1.05 * x.max
    else
      .95 * x.max

  return(c(fake.x.min, fake.x.max))
}


#' Similar to summary() but with the return values being:
#' "  N"  "Min."  "1st Qu." "Median"  "3rd Qu." "Max." "Mean"  "S.D."
#'
#' @param x Sorted list of x-values in the dataset.
#' @param pr A logical unit to indicate whether to display the summary information on the graph.
#' @param xat The x positions of the summary values.
#' @param yat The y positions of the summary values.
#' @param digits The number of floating point digits the summary values are rounded to.
#' @param adj One or two values in [0, 1] which specify the x (and optionally y) adjustment of the labels.
#' @param cex A numerical value to specify the size of the text.
#' @param ... Further graphical parameters (from par), such as srt, family and xpd.
#'
#' @return A vector of summary values: N, Min, Q1, Median, Q3, Max, Mean, and SD.
summ <- function(x, pr = F, xat = NA, yat = c(.1, .06), digits = 4, adj = 0, cex = par()$cex, ...) {
  x <- as.vector(x)
  y <- x[!is.na(x)]

  temp1 <- summary(y)
  temp2 <- sqrt(var(y))
  ans <- c(length(y), temp1[1:3], temp1[5:6], temp1[4], temp2)
  names(ans) <- c("  N", "Min.", "1st Qu.", "Median", "3rd Qu.", "Max.", "Mean", "S.D.")

  if (pr) {
    if (is.na(xat[1])) {
      xat <- seq(range(y, na.rm = T)[1], range(y, na.rm = T)[2], length = 8)
    }
    text(xat, yat[1], paste(names(ans)),adj = adj, cex = cex, ...)
    text(xat, yat[2], paste(round(ans, digits)), adj = adj, cex = cex, ...)
  }
  return(round(ans, digits))
}
