#' Draws a standard boxplot according to user specifications.
#'
#' @param x Sorted list of x-values in the dataset.
#' @param nvectors A numeric value indicating the number of groupings in the dataset.
#' @param lower.edge The first value from graph.region indicating the lower graphical region.
#' @param upper.edge The second value from graph.region indicating the upper graphical region.
#' @param xlim A numeric vector consists of 2 elements which indicates the range of x-axis.
#' @param line.extend.col A numeric value to indicate the color of the line.
#' @param line.extend.width A numeric value to indicate the thickness of the line.
#' @param box.density A numeric value to indicate the density of the box shading lines.
#' @param box.angle A numeric value to indicate the angle of the box shading lines.
#' @param box.col A vector to indicate the color(s) of the box shading lines.
#' @param point.cex A numeric value to specify the size of points
#' @param point.pch A numeric or character value to specify the shape of points.
#' @param point.col A numeric value to specify the color of the points.
#' @param axes A logical value to indicate whether or not to plot axes.
#' @param xaxt "s"=standard x-axis plotting,"n"=null x-axis plotting.
#' @param xlab Labels on x-axis.
bp <- function(x, nvectors, lower.edge, upper.edge, xlim, line.extend.col,
               line.extend.width, box.density, box.angle, box.col, point.cex,
               point.pch, point.col, axes, xaxt, xlab) {

  # Reset some variables ------------------------------------------------------
  box.p <- c(.25, .5, .75)
  graph.width <- "f"
  graph.type  <- "c"
  point.pattern <- "on-line"
  n <- vector()

  # Draw frame ----------------------------------------------------------------
  if (all(is.na(xlim))) {
    x.min <- min(sapply(x, min))
    x.max <- max(sapply(x, max))
    fake.x <-maxmin.frame(x.min, x.max)
  } else {
    fake.x <- xlim
  }
  plot(fake.x, c(0, 1), type = "n", axes = F, ylab = "", xlab = xlab)

  if (axes) {
    axis(1, xaxt = xaxt)
    box()
  }

  # Determine the height of the box -------------------------------------------
  for (k in 1:nvectors) {
    n[k] <- length(x[[k]])
    low <- (k - 1) / nvectors + (1 / nvectors) * lower.edge
    up  <- (k - 1) / nvectors + (1 / nvectors) * upper.edge
    boxwidth  <- up - low
    boxcenter <- low + boxwidth / 2
    q  <- quantile(x[[k]], probs = box.p)
    nq <- length(q)
    loweredge <- rep(low, nq)
    upperedge <- rep(up, nq)

    boxcolor <-
      if (k %% length(box.col))
        box.col[k %% length(box.col)]
      else
        box.col[length(box.col)]

    # Draw the box ------------------------------------------------------------
    polygon(c(q, rev(q)), c(upperedge, rev(loweredge)), density = box.density,
            col = boxcolor, angle = box.angle, lwd = line.extend.width, border = F)
    segments(q, loweredge, q, upperedge, lwd = line.extend.width, col = line.extend.col)
    lines(q, loweredge, lwd = line.extend.width, col = line.extend.col)
    lines(q, upperedge, lwd = line.extend.width, col = line.extend.col)

    # Draw lines --------------------------------------------------------------
    qr <- quantile(x[[k]], probs = c(.25, .75))
    xlines  <- c(max((qr[1] - 1.5 * (qr[2] - qr[1])), min(x[[k]])), qr[1], NA, qr[2],
                 min((qr[2] + 1.5 * (qr[2] - qr[1])), max(x[[k]])))
    nxlines <- length(xlines)
    lines(xlines, rep(boxcenter, nxlines), lwd = line.extend.width, col = line.extend.col)
    xpoints <- c(x[[k]][x[[k]] < xlines[1]], x[[k]][x[[k]] > xlines[nxlines]])

    # Draw points -------------------------------------------------------------
    points(xpoints, rep(boxcenter, length(xpoints)), pch = point.pch,
           cex = point.cex, col = point.col)
  }
}

#' Draws a standard histogram according to user specifications.
#'
#' @param x Sorted list of x-values in the dataset.
#' @param nvectors A numeric value indicating the number of groupings in the dataset.
#' @param graph.uniform A logical value(T/F) to indicate the choice of uniform/non-uniform height on y-axis across groups.
#' @param low A numerical vector indicating the lower edge of each box.
#' @param boxwidth A numerical vector indicating the width of each box.
#' @param nclass The smoothing parameter which corresponds to how many classes the total number of observations will be divided.
#' @param box.label.cex A numeric value to control the size of the labels.
#' @param box.label.above A numeric value indicating the distance of the labels above the box.
#' @param box.label.on A Logical value indicating whether to label the percentiles on the boxes.
#' @param xlim A numeric vector consists of 2 elements which indicates the range of x-axis.
#' @param line.extend.col A numeric value to indicate the color of the line.
#' @param line.extend.width A numeric value to indicate the thickness of the line.
#' @param box.density A numeric value to indicate the density of the box shading lines.
#' @param box.angle A numeric value to indicate the angle of the box shading lines.
#' @param box.col A vector to indicate the color(s) of the box shading lines.
#' @param axes A logical value to indicate whether or not to plot axes.
#' @param xaxt "s"=standard x-axis plotting,"n"=null x-axis plotting.
#' @param xlab Labels on x-axis.
#' @param breaks A numeric vector indicating break points for histogram and polygon.
#' @param x.min A numeric value indicating the smallest break point.
#' @param x.max A numeric value indicating the largest break point.
histo <- function(x, nvectors, graph.uniform, low, boxwidth, nclass,
                  box.label.cex, box.label.above, box.label.on, xlim,
                  line.extend.col, line.extend.width, box.density, box.angle,
                  box.col, axes, xaxt, xlab, breaks, x.min, x.max) {

  # Draw frame ----------------------------------------------------------------
  fake.x <-
    if (all(is.na(xlim)))
      maxmin.frame(x.min, x.max)
    else
      xlim
  plot(fake.x, c(0, 1), type = "n", axes = F, ylab = "", xlab = xlab)
  if (axes) {
    axis(1, xaxt = xaxt)
    box()
  }

  # Determine the frequency in each interval ----------------------------------
  freq <- list()
  line <- list()
  qlines <- list()
  freqtext  <- list()
  midpoints <- list()

  for (k in 1:nvectors) {
    intervals <-
      if (all(is.na(breaks)) && graph.uniform)
        seq(x.min, x.max, by = (x.max - x.min) / nclass)
      else if (all(is.na(breaks)) && !graph.uniform)
        seq(min(x[[k]]), max(x[[k]]), by = (max(x[[k]]) - min(x[[k]])) / nclass)
      else
        breaks

    nintervals <- length(intervals)
    freq[[k]] <- as.vector(table(cut(x[[k]],
                                     breaks = c(intervals[1] - .1,
                                                intervals[2:(nintervals - 1)],
                                                intervals[nintervals] + .1))))

    freqtext[[k]] <- freq[[k]]
    freqtext[[k]][freq[[k]] == 0] <- NA
  }
    max.freq <- max(sapply(freq, max))

  for (k in 1:nvectors) {
    line[[k]] <- matrix(0, (nintervals - 1), 2)
    line[[k]][, 1] <- intervals[1:(nintervals - 1)]
    line[[k]][, 2] <- intervals[2:nintervals]
    midpoints[[k]] <- line[[k]][, 1] + (line[[k]][, 2] - line[[k]][, 1]) / 2
    upperedge <- low[k] + (freq[[k]] / max.freq) * boxwidth[k]
    loweredge <- rep(low[k], 2)

    boxcolor <-
      if (k %% length(box.col))
        box.col[k %% length(box.col)]
    else
      box.col[length(box.col)]

    for (i in 1:(nintervals - 1)) {
      polygon(c(line[[k]][i, 1], line[[k]][i, 1], line[[k]][i, 2], line[[k]][i, 2]),
              c(loweredge[1], upperedge[i], upperedge[i], loweredge[1]), border = F,
              density = box.density, angle = box.angle, col = boxcolor, lwd = line.extend.width)
      segments(line[[k]][i, 1], loweredge[1], line[[k]][i, 1], upperedge[i],
               lwd = line.extend.width, col = line.extend.col)
      segments(line[[k]][i, 2], loweredge[1], line[[k]][i, 2], upperedge[i],
               lwd = line.extend.width, col = line.extend.col)
      lines(line[[k]][i, 1:2], c(upperedge[i], upperedge[i]),
            lwd = line.extend.width, col = line.extend.col)
    }

    if (graph.uniform)
      lines(c(x.min, x.max), loweredge, lwd = line.extend.width, col = line.extend.col)
    else
      lines(c(min(x[[k]])), max(x[[k]]), c(loweredge[1], upperedge[1]),
            lwd = line.extend.width, col = line.extend.col)
    if (box.label.on) {
      text(midpoints[[k]], upperedge + box.label.above, freqtext[[k]], cex = box.label.cex)
    }
  }
}

#' Draws a standard polygon according to user specifications.
#'
#' @param x Sorted list of x-values in the dataset.
#' @param nvectors A numeric value indicating the number of groupings in the dataset.
#' @param graph.uniform A logical value(T/F) to indicate the choice of uniform/non-uniform height on y-axis across groups.
#' @param low A numerical vector indicating the lower edge of each box.
#' @param boxwidth A numerical vector indicating the width of each box.
#' @param nclass The smoothing parameter which corresponds to how many classes the total number of observations will be divided.
#' @param box.label.cex A numeric value to control the size of the labels.
#' @param box.label.above A numeric value indicating the distance of the labels above the box.
#' @param box.label.on A Logical value indicating whether to label the percentiles on the boxes.
#' @param xlim A numeric vector consists of 2 elements which indicates the range of x-axis.
#' @param line.extend.col A numeric value to indicate the color of the line.
#' @param line.extend.width A numeric value to indicate the thickness of the line.
#' @param box.density A numeric value to indicate the density of the box shading lines.
#' @param box.angle A numeric value to indicate the angle of the box shading lines.
#' @param box.col A vector to indicate the color(s) of the box shading lines.
#' @param axes A logical value to indicate whether or not to plot axes.
#' @param xaxt "s"=standard x-axis plotting,"n"=null x-axis plotting.
#' @param xlab Labels on x-axis.
#' @param breaks A numeric vector indicating break points for histogram and polygon.
#' @param x.min A numeric value indicating the smallest break point.
#' @param x.max A numeric value indicating the largest break point.
polygn <- function(x, nvectors, graph.uniform, low, boxwidth, nclass,
                 box.label.cex, box.label.above, box.label.on, xlim,
                 line.extend.col, line.extend.width, box.density, box.angle,
                 box.col, axes, xaxt, xlab, breaks, x.min, x.max) {

  # Determine frequency in each interval --------------------------------------
  freq <- list()
  line <- list()
  qlines <- list()
  midpoints <- list()
  xpoints <- list()
  ypoints <- list()
  loweredge <- list()
  upperedge <- list()

  for (k in 1:nvectors) {
    x.min <- min(sapply(x, min))
    x.max <- max(sapply(x, max))
    intervals <-
      if (all(is.na(breaks)) && graph.uniform)
        seq(x.min, x.max, by = (x.max - x.min) / nclass)
      else if (all(is.na(breaks)) && !graph.uniform)
        seq(min(x[[k]]), max(x[[k]]), by = (max(x[[k]]) - min(x[[k]])) / nclass)
      else
        breaks
    nintervals <- length(intervals)
    freq[[k]] <- as.vector(table(cut(x[[k]],
                                     breaks = c(intervals[1] - .1,
                                                intervals[2:(nintervals - 1)],
                                                intervals[nintervals] + .1))))
  }
  max.freq <- max(sapply(freq, max))
  for (k in 1:nvectors) {
    line[[k]] <- matrix(0, (nintervals - 1), 2)
    line[[k]][, 1] <- intervals[1:(nintervals - 1)]
    line[[k]][, 2] <- intervals[2:nintervals]
    qlines[[k]] <- c(line[[k]][1, 1] - (line[[k]][1, 2] - line[[k]][1, 1]) / 2,
                     line[[k]][(nintervals - 1), 2] + (line[[k]][1, 2] - line[[k]][1, 1]) / 2)
    midpoints[[k]] <- line[[k]][, 1] + (line[[k]][, 2] - line[[k]][, 1]) / 2
    upperedge[[k]] <- low[k] + (freq[[k]] / max.freq) * boxwidth[k]
    loweredge[[k]] <- rep(low[k], 2)
    xpoints[[k]] <- c(line[[k]][1, 1] - (line[[k]][1, 2] - line[[k]][1, 1]) / 2,
                      midpoints[[k]], line[[k]][(nintervals - 1), 2] +
                        (line[[k]][1, 2] - line[[k]][1, 1]) / 2)
    ypoints[[k]] <- c(low[k], upperedge[[k]], low[k])
  }

  # Draw frame and polygon ----------------------------------------------------
  fake.x <-
    if (all(is.na(xlim)))
      c(min(sapply(qlines, min)), max(sapply(qlines, max)))
    else
      xlim
  plot(fake.x, c(0, 1), type = "n", axes = F, ylab = "", xlab = xlab)

  if (axes) {
    axis(1, xaxt = xaxt)
    box()
  }


  for (k in 1:nvectors) {
    polygon(xpoints[[k]], ypoints[[k]], density = box.density, angle = box.angle,
            col = box.col, lwd = line.extend.width, border = F)
    lines(xpoints[[k]], ypoints[[k]], lwd = line.extend.width, col = line.extend.col)
    lines(c(xpoints[[k]][1], xpoints[[k]][(nintervals - 1) + 2]), loweredge[[k]],
          lwd = line.extend.width, col = line.extend.col)
    if (box.label.on) {
      text(midpoints[[k]], upperedge[[k]] + box.label.above, freqtext[[k]], cex = box.label.cex)
    }
  }
}
